/* GDBX^u@@ˑ֐Q*/

#include "gdb-stub-common.h"
#include <gdb-stub.h>

int registers[NUMREGS];
static const char hexchars[] = "0123456789abcdef";
char remcomInBuffer[BUFMAX];
char remcomOutBuffer[BUFMAX];

char * strcpy(char *dest, const char *src){

  while(*src != '\0'){
    *dest = *src;
    src ++;
    dest ++;
  }
  *dest = 0;
  return dest;
}

char highhex(int  x)
{
  return hexchars[(x >> 4) & 0xf];
}

char lowhex(int  x)
{
  return hexchars[x & 0xf];
}

/*
 * Routines to handle hex data
 */

static int
hex (char ch)
{
  if ((ch >= 'a') && (ch <= 'f'))
    return (ch - 'a' + 10);
  if ((ch >= '0') && (ch <= '9'))
    return (ch - '0');
  if ((ch >= 'A') && (ch <= 'F'))
    return (ch - 'A' + 10);
  return (-1);
}

/* convert the memory, pointed to by mem into hex, placing result in buf */
/* return a pointer to the last char put in buf (null) */
static char *
mem2hex (char *mem, char *buf, int count)
{
  int i;
  int ch;
  for (i = 0; i < count; i++)
    {
      ch = *mem++;
      *buf++ = highhex (ch);
      *buf++ = lowhex (ch);
    }
  *buf = 0;
  return (buf);
}

/* convert the hex array pointed to by buf into binary, to be placed in mem */
/* return a pointer to the character after the last byte written */

static char *
hex2mem (char *buf, char *mem, int count)
{
  int i;
  unsigned char ch;
  for (i = 0; i < count; i++)
    {
      ch = hex (*buf++) << 4;
      ch = ch + hex (*buf++);
      *mem++ = ch;
    }
  return (mem);
}

static int
hexToInt (char **ptr, int *intValue)
{
  int numChars = 0;
  int hexValue;

  *intValue = 0;

  while (**ptr)
    {
      hexValue = hex (**ptr);
      if (hexValue >= 0)
	{
	  *intValue = (*intValue << 4) | hexValue;
	  numChars++;
	}
      else
	break;

      (*ptr)++;
    }

  return (numChars);
}

char *getpacket (void)
{
  unsigned char *buffer = &remcomInBuffer[0];
  unsigned char checksum;
  unsigned char xmitcsum;
  int count;
  char ch;

  while (1)
    {
      /* wait around for the start character, ignore all other characters */
      while ((ch = getDebugChar ()) != '$')
	;

    retry:
      checksum = 0;
      xmitcsum = -1;
      count = 0;

      /* now, read until a # or end of buffer is found */
      while (count < BUFMAX)
	{
	  ch = getDebugChar ();
          if (ch == '$')
            goto retry;
	  if (ch == '#')
	    break;
	  checksum = checksum + ch;
	  buffer[count] = ch;
	  count = count + 1;
	}
      buffer[count] = 0;
      if (ch == '#')
	{
 	  ch = getDebugChar ();
 	  xmitcsum = hex (ch) << 4;
 	  ch = getDebugChar ();
 	  xmitcsum += hex (ch);

	  if (checksum != xmitcsum)
	    {
	      putDebugChar ('-');	/* failed checksum */
	    }
	  else
	    {
	      putDebugChar ('+');	/* successful transfer */

	      /* if a sequence char is present, reply the sequence ID */
	      if (buffer[2] == ':')
		{
		  putDebugChar (buffer[0]);
		  putDebugChar (buffer[1]);

 		  return &buffer[3];
		}

	      return &buffer[0];
	    }
	}
    }
}


/* send the packet in buffer. */
static void
putpacket (unsigned char *buffer)
{
  unsigned char checksum;
  int count;
  unsigned char ch;

  /*  $<packet info>#<checksum>. */
  do
    {
      putDebugChar('$');
      checksum = 0;
      count = 0;

      while (ch = buffer[count])
	{
	  putDebugChar(ch);
	  checksum += ch;
	  count += 1;
	}

      putDebugChar('#');
      putDebugChar(hexchars[checksum >> 4]);
      putDebugChar(hexchars[checksum & 0xf]);

    }
  while (getDebugChar() != '+');
}

void
gdb_handle_exception (int exceptionVector)
{
  int sigval;
  int addr, length;
  char *ptr;
  int i, j;
  char regnum_char[5];
  int regnum;
  unsigned int regval;

  disable_int();

  /* reply to host that an exception has occurred */
  sigval = computeSignal (exceptionVector);
  if(sigval > 0){
    remcomOutBuffer[0] = 'S';
    remcomOutBuffer[1] = highhex(sigval);
    remcomOutBuffer[2] = lowhex (sigval);
    remcomOutBuffer[3] = 0;

    putpacket (remcomOutBuffer);
  }

  remcomOutBuffer[0] = 0;
  
  while (1)
    {
      remcomOutBuffer[0] = 0;
      ptr = getpacket ();


      switch (*ptr++)
	{
	case 'H':
	case '!':
	  strcpy (remcomOutBuffer, "OK");
	  break;
	case '?':
	  remcomOutBuffer[0] = 'S';
	  remcomOutBuffer[1] = highhex (sigval);
	  remcomOutBuffer[2] = lowhex (sigval);
	  remcomOutBuffer[3] = 0;
	  break;
	case 'g':		/* return the value of the CPU registers */
	  mem2hex ((char *) registers, remcomOutBuffer, NUMREGBYTES);
	  break;
	case 'G':		/* set the value of the CPU registers - return OK */
	  hex2mem (ptr, (char *) registers, NUMREGBYTES);
	  strcpy (remcomOutBuffer, "OK");
	  break;

	case 'P':		/* set the value of the CPU registers with index- return OK */
#ifdef ARM
	  // ARM1̂ݑĂ
          regnum = hex(*ptr);
          ptr++;
          ptr++;
#else
	  regnum = hex(*ptr) << 4;
	  ptr++;
	  regnum += hex(*ptr);
	  ptr++;
	  ptr++;
#endif
	  hex2mem (ptr, (char *)&regval, 4);
	  registers[regnum] = regval;
	  strcpy (remcomOutBuffer, "OK");
	  break;

	  /* mAA..AA,LLLL  Read LLLL bytes at address AA..AA */
	case 'm':
	  /* TRY, TO READ %x,%x.  IF SUCCEED, SET PTR = 0 */
	  if (hexToInt (&ptr, &addr))
	    if (*(ptr++) == ',')
	      if (hexToInt (&ptr, &length))
		{
		  ptr = 0;
		  mem2hex ((char *) addr, remcomOutBuffer, length);
		}
	  if (ptr)
	    strcpy (remcomOutBuffer, "E01");
	  break;
	  /* MAA..AA,LLLL: Write LLLL bytes at address AA.AA return OK */
	case 'M':
	  /* TRY, TO READ '%x,%x:'.  IF SUCCEED, SET PTR = 0 */
	  if (hexToInt (&ptr, &addr)){
	    if (*(ptr++) == ',')
	      if (hexToInt (&ptr, &length))
		if (*(ptr++) == ':')
		  {
#ifdef CORTEX_M3
		    if(addr >= 0x20000000 && addr < 0x20005000){// SRAM珑
#endif	    
		      hex2mem (ptr, (char *) addr, length);
		      ptr = 0;
		      strcpy (remcomOutBuffer, "OK");
#ifdef CORTEX_M3
		    }
		    else
		      strcpy (remcomOutBuffer, "E02");
		    
#endif	    
		  }
	  }
	  if (ptr)
	    strcpy (remcomOutBuffer, "E02");
	  
	  break;
	  
	  /* cAA..AA    Continue at address AA..AA(optional) */
	  /* sAA..AA   Step one instruction from AA..AA(optional) */
	case 's':
	case 'c':
	  enable_int();
	  return;
	case 'Z': // u[N|Cg̐ݒ
#ifdef SUPPORT_Z0_COMMAND
	  if(*ptr == '0'){//\tgEFAu[N|Cg
	    ptr ++;
	    if(*ptr == ','){
	      ptr ++;
	      if(hexToInt (&ptr, &addr)){
		if(*ptr == ','){
		  volatile unsigned int *bkpt_addr;
		  // length͖B4oCĝ͂
		  // u[N|Cg̋󂫂T
		  int i_break;
		  for(i_break=0;i_break<MAX_BREAK;i_break++)
		    if(breakpoint_addr_list[i_break].used == 0)
		      break;
		  if(i_break == MAX_BREAK){
		    strcpy (remcomOutBuffer, "E00");
		    break;
		  }
		  // 󂫂𔭌
		  // w肳ꂽAhXɖ`߂
		  breakpoint_addr_list[i_break].addr = addr;
		  bkpt_addr = (unsigned int *)addr;
		  // {̖߂obNAbv
		  breakpoint_addr_list[i_break].inst = *bkpt_addr;
		  *bkpt_addr = 0xFC000000; // `߂
		  // gptOZbg
		  breakpoint_addr_list[i_break].used = 1;

		  strcpy (remcomOutBuffer, "OK");
		}
	      }
	    }
	    break;
	  }
#endif /*SUPPORT_Z0_COMMAND*/
#ifdef SUPPORT_Z1_COMMAND
	  if(*ptr == '1'){//n[hEFAu[N|Cg
	    ptr ++;
	    if(*ptr == ','){
	      ptr ++;
	      if(hexToInt (&ptr, &addr)){
		if(*ptr == ','){
		  unsigned int *bkpt_addr;
		  if(UBC_set(addr))
		    strcpy (remcomOutBuffer, "OK");
		  else
		    strcpy (remcomOutBuffer, "E03");
		}
	      }
	    }
	  }
#endif /*SUPPORT_Z1_COMMAND*/
	  break;
	case 'z': // u[N|Cg̍폜
#ifdef SUPPORT_Z0_COMMAND
	  if(*ptr == '0'){//\tgEFAEFAu[N|Cg
	    ptr ++;
	    if(*ptr == ','){
	      ptr ++;
	      if(hexToInt (&ptr, &addr)){
		if(*ptr == ','){
		  volatile unsigned int *bkpt_addr;
		  // AhXvu[N|CgT
		  int i_break;
		  for(i_break=0;i_break<MAX_BREAK;i_break++)
		    if((breakpoint_addr_list[i_break].used == 1) &&
		       (breakpoint_addr_list[i_break].addr == addr))
		      break;
		  if(i_break == MAX_BREAK){ // Ȃ
		    strcpy (remcomOutBuffer, "E00");
		    break;
		  }
		  // u[N𖳌
		  breakpoint_addr_list[i_break].used = 0;

		  bkpt_addr = (unsigned int *)(breakpoint_addr_list[i_break].addr);
		  breakpoint_addr_list[i_break].addr = 0;
		  *bkpt_addr = breakpoint_addr_list[i_break].inst;
		  breakpoint_addr_list[i_break].inst = 0;
		  strcpy (remcomOutBuffer, "OK");
		}
	      }
	    }
	    break;
	  }
#endif /*SUPPORT_Z0_COMMAND*/
#ifdef SUPPORT_Z1_COMMAND
	  if(*ptr == '1'){//n[hEFAu[N|Cg
	    ptr ++;
	    if(*ptr == ','){
	      ptr ++;
	      if(hexToInt (&ptr, &addr)){
		if(*ptr == ','){
		  UBC_clr(addr);
		  strcpy (remcomOutBuffer, "OK");
		}
	      }
	    }
	  }
	  break;
#endif /*SUPPORT_Z1_COMMAND*/
	case 'k':
	  gdb_stub_kill_command();
	case 'R':		/* do nothing */
	case 'r':
	  goto noreply;
	default:
	  break;
	}			/* switch */

      /* reply to the request */
      putpacket (remcomOutBuffer);
    noreply:
      continue;
    }
  return;
}


// ̊֐Oɂ΂AGDBɐԂB
void handle_exception(int exceptionVector)
{
  gdb_handle_exception (exceptionVector);
}

extern unsigned int target_main_addr;

// j^main[`
int main(){
  unsigned int *bkpt_addr, *ilegal_addr;
  volatile int msr_val;
  int i,j;
  int sigval;
  void (*target_main)();

  //asm("BL target_main");
  //  asm("LDR R0, =0x08005101");
  //  asm("BX R0");

  cpu_init_at_main();

  init_serial();

  for(i=0;i<NUMREGS;i++)
    registers[i] = 0;
    
  sigval = NO_EXCEPTION;
  while(1){
    gdb_handle_exception(sigval);

    // _E[hvÕGg[|CgĂяoB
#ifndef CORTEX_M3
    target_main = (void *)registers[PC];
#endif //CORTEX_M3
    enable_int();

#ifdef CORTEX_M3
    asm("LDR R0, =target_main_addr");
    asm("LDR R1, =0x1");
    asm("ORR R0, R1");
    asm("BX R0");
#endif
  
#ifndef CORTEX_M3
    target_main();
#endif
    sigval  = TERM_EXCEPTION;
  }
  return 1;
}





